/***********
 Simple query execution engine
 
 Written in Java/JDK 1.1
 For CSE 544 Homework 2
 ***********/

import java.io.*;
import java.util.Date;

public class QueryExec {

	/** Main code - input a query plan and execute it, outputting
	    results to output.dat */
	public void exec() {
        Tuple t;
        PrintWriter d = null;
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        
        System.out.println("CSE 544 Homework 2, Spring 1999\n");
        
        String query = "";

        System.out.println("Please enter your query plan: ");
        try {
            query = in.readLine();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        /* Only try to execute if we have some sort of query */
        if (!query.equals("")) {
            PlanParser p = new PlanParser();
            Operator root = p.parse(query);
            
            if (root == null) {
                System.err.println("Error parsing query.");
                System.exit(1);
            }

            long start = new Date().getTime();
            
            // Now send open message to root, which percolates it
            int count = 0;
            if (!root.open()) {
                System.err.println("Unable to open query plan");
                System.exit(1);
            }

            // Set up output file and print headers
            try {
                d = new PrintWriter(new BufferedWriter(new FileWriter("output.dat")));
                for (int index = 0; index < root.labels().size(); index++)
                    d.print(root.labels().elementAt(index) + "\t");
                d.println();    
            } catch (java.io.IOException i) {
                i.printStackTrace();
                System.exit(1);
            }
            
            // Main loop - call next(), get tuple, output it
            while ((t = root.next()) != null) {
                    for (int index = 0; index < t.data().size(); index++)
                        d.print(t.data().elementAt(index) + "\t");
                    d.println();

                    // Call Java garbage collector every so often
                    if (count % 1000 == 0)
                        System.gc();
                        
                    count++;
            }    

            // Done - close file
            d.close();
            
            // and root operator
            if (!root.close())
                System.err.println("Error closing plan.");
                
            long end = new Date().getTime();

            System.out.println("Query execution complete.  Wrote " + count + " tuples in " + (end - start) + "ms.");
        }    
	}

    static public void main(String args[]) {
        QueryExec s = new QueryExec();
        
        s.exec();
    }

}

