/**********
 Scan operator
 
 Simple query execution engine
 For CSE 544 Homework 2
 **********/

import java.io.*;
import java.util.Vector;

/** Scan operator - read data from a file */
public class Scan implements Operator {
    ObjectInputStream _if;      // Input file being scanned
    String _filename;           // Its name
    boolean _isOpen;            // Operator-open status
    Vector _labels;             // Metadata about attributes

    /** Constructor - just save filename and clear open status */
    public Scan(String filename) {
        _filename = filename + ".tbl";
        _isOpen = false;
    }
    
    /** Open file, read metadata */
    public boolean open() {
        System.out.println("Opening scan of " + _filename);

        try {
            _if = new ObjectInputStream(new BufferedInputStream(new FileInputStream(_filename), 65536));
        } catch (Exception e) {
            System.err.println("Error reading " + _filename);
            return false;
        }

        _isOpen = true;
 
        try {
            _labels = (Vector)_if.readObject();
        } catch (Exception cf) {
            System.err.println("Error reading metadata");
            return false;
        }
        return true;
    }
    
    /** Next operator - get a tuple from the file */
    public Tuple next() {
        try {
            Vector data = (Vector)_if.readObject();
    
            // If you want to see what's going on...
            //System.out.println(data.elementAt(0));
            
            return new Tuple(_labels, data);

        } catch (ClassNotFoundException cn) {
            System.err.println("Could not find class");
            return null;
        } catch (EOFException eof) {
            // Done if we're here
            return null;
        } catch (IOException e) {
            System.err.println("Error reading " + _filename);
            return null;
        }
    }
    
    /** Close - close file */
    public boolean close() {
        System.out.println("Closing scan");
        
        try {
            _if.close();
        } catch (IOException e) {
            System.err.println("Error reading " + _filename);
            return false;
        }
        
        _isOpen = false;
        return true;
    }
    
    /** Return metadata info on attribute labels */
    public Vector labels() { return _labels; }
    
    /** Operator-open status */
    public boolean isOpen() { return _isOpen; }
}